package com.mygdx.game.screen;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.Box2DDebugRenderer;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.World;
import com.mygdx.game.level.Level;
import java.io.IOException;

/**
 *
 * @author Martin
 */
public class GameScreen extends ScreenAdapter {

	public static final float WORLD_TIMESTEP = 1 / 60f;
	public static final int WORLD_VELC_ITER = 6;
	public static final int WORLD_POS_ITER = 2;

	private OrthographicCamera camera;
	private SpriteBatch spriteBatch;
	private TextureAtlas charAtlas;
	private TextureAtlas tilesAtlas;
	private BitmapFont font;
	private ShapeRenderer shapeRenderer;
	private Animation lumFlyAnimation;
	private float lumFlyStateTime;
	private Sprite lumSprite;
	private Sprite lumSprite2;
	private Level currLevel;
	private final static int TILE_SIZE = 42;
	private World world;
	private Box2DDebugRenderer debugRenderer;
	private Body lumBody;
	private Body lumBody2;
	

	public GameScreen() {
		// Zeichenflaeche initen
		shapeRenderer = new ShapeRenderer();

		camera = new OrthographicCamera();
		camera.setToOrtho(false, 1024, 768);

		spriteBatch = new SpriteBatch();

		// LUM Animation
		charAtlas = new TextureAtlas(Gdx.files.internal("char.atlas"));
		lumFlyAnimation = new Animation(0.14f,
				charAtlas.findRegion("frame-1"),
				charAtlas.findRegion("frame-2"),
				charAtlas.findRegion("frame-3"),
				charAtlas.findRegion("frame-4")
		);
		lumFlyAnimation.setPlayMode(Animation.PlayMode.LOOP);
		lumFlyStateTime = 0;
		lumSprite = new Sprite(charAtlas.findRegion("frame-1"));
		lumSprite.setPosition(100, 200);
		lumSprite.setColor(Color.RED);
		lumSprite.setSize(120, 90);
		
		lumSprite2 = new Sprite(charAtlas.findRegion("frame-1"));
		lumSprite2.setPosition(100, 200);
		lumSprite2.setColor(Color.BLUE);
		lumSprite2.setSize(120, 90);

		tilesAtlas = new TextureAtlas(Gdx.files.internal("tiles.atlas"));

		FreeTypeFontGenerator freeTypeFontGenerator = new FreeTypeFontGenerator(
				Gdx.files.internal("Gravity-Regular.otf"));

		FreeTypeFontGenerator.FreeTypeFontParameter para
				= new FreeTypeFontGenerator.FreeTypeFontParameter();
		para.size = 36;
		para.color = Color.WHITE;

		font = freeTypeFontGenerator.generateFont(para);

		world = new World(new Vector2(0, -30f), true);

		setupLevel();
		setupLUM();
		debugRenderer = new Box2DDebugRenderer();
	}

	private void setupLevel() {
		try {
			currLevel = Level.loadFromFile(Gdx.files.internal("level1.txt"));

			int[][] levelTiles = currLevel.getTiles();
			for (int i = 0; i < levelTiles.length; i++) {
				for (int j = 0; j < levelTiles[i].length; j++) {
					Vector2 posTile = new Vector2(
							j * TILE_SIZE + (TILE_SIZE / 2),
							i * TILE_SIZE + (TILE_SIZE / 2));
					switch (levelTiles[i][j]) {
						case Level.GRASS_TYPE:
							BodyDef tileDef = new BodyDef();
							tileDef.type = BodyDef.BodyType.StaticBody;
							tileDef.position.set(posTile);

							Body tileBody = world.createBody(tileDef);
							PolygonShape groundBox = new PolygonShape();
							groundBox.setAsBox(TILE_SIZE / 2,
									TILE_SIZE / 2);
							tileBody.createFixture(groundBox, 0.0f);
							groundBox.dispose();
							break;
					}
				}
			}
		} catch (IOException ex) {
			Gdx.app.log("LevelLoad", ex.toString());
		}
	}

	private void setupLUM() {
		BodyDef lumBodyDef = new BodyDef();
		lumBodyDef.type = BodyDef.BodyType.DynamicBody;
		lumBodyDef.position.set(100, 200);
		lumBodyDef.fixedRotation = true;

		lumBody = world.createBody(lumBodyDef);

		PolygonShape lumBodyBox = new PolygonShape();
		lumBodyBox.setAsBox(60, 45);
		lumBody.createFixture(lumBodyBox, 0.0005f);
		lumBodyBox.dispose();

		lumBody2 = world.createBody(lumBodyDef);

		PolygonShape lumBody2Box = new PolygonShape();
		lumBody2Box.setAsBox(60, 45);
		lumBody2.createFixture(lumBodyBox, 0.0005f);
		lumBody2Box.dispose();
	}

	@Override
	public void render(float delta) {
		world.step(WORLD_TIMESTEP, WORLD_VELC_ITER, WORLD_POS_ITER);

		lumFlyStateTime += delta;

		float force = 30;
		if (Gdx.input.isKeyPressed(Input.Keys.LEFT)) {
			lumBody.applyLinearImpulse(
					new Vector2(-force, 0),
					lumBody.getPosition(), true);
		}
		if (Gdx.input.isKeyPressed(Input.Keys.RIGHT)) {
			lumBody.applyLinearImpulse(
					new Vector2(force, 0),
					lumBody.getPosition(), true);
		}
		if (Gdx.input.isKeyPressed(Input.Keys.SPACE)) {
			lumBody.applyLinearImpulse(
					new Vector2(0, force / 5),
					lumBody.getPosition(), true);
		}
		if (Gdx.input.isKeyPressed(Input.Keys.A)) {
			lumBody2.applyLinearImpulse(
					new Vector2(-force, 0),
					lumBody.getPosition(), true);
		}
		if (Gdx.input.isKeyPressed(Input.Keys.D)) {
			lumBody2.applyLinearImpulse(
					new Vector2(force, 0),
					lumBody.getPosition(), true);
		}
		if (Gdx.input.isKeyPressed(Input.Keys.W)) {
			lumBody2.applyLinearImpulse(
					new Vector2(0, force / 5),
					lumBody.getPosition(), true);
		}

		Gdx.gl.glClearColor(1, 1, 1, 1);
		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

		spriteBatch.setProjectionMatrix(camera.combined);
		spriteBatch.begin();

		font.setColor(Color.BLACK);
		font.draw(spriteBatch, "LUM (Linux User Monster)", 100, 300);

		Vector2 lumPos = lumBody.getPosition();
		lumSprite.setCenter(lumPos.x, lumPos.y);
		lumSprite.setRotation(MathUtils.radiansToDegrees * lumBody.getAngle());

		lumSprite.setRegion(lumFlyAnimation.getKeyFrame(lumFlyStateTime));
		if (lumBody.getLinearVelocity().x >= 0) {
			lumSprite.setFlip(false, false);
		} else {
			lumSprite.setFlip(true, false);
		}

		lumSprite.draw(spriteBatch);
		
		Vector2 lumPos2 = lumBody2.getPosition();
		lumSprite2.setCenter(lumPos2.x, lumPos2.y);
		lumSprite2.setRotation(MathUtils.radiansToDegrees * lumBody.getAngle());

		lumSprite2.setRegion(lumFlyAnimation.getKeyFrame(lumFlyStateTime));
		if (lumBody2.getLinearVelocity().x >= 0) {
			lumSprite2.setFlip(false, false);
		} else {
			lumSprite2.setFlip(true, false);
		}

		lumSprite2.draw(spriteBatch);

		int[][] levelTiles = currLevel.getTiles();
		for (int i = 0; i < levelTiles.length; i++) {
			for (int j = 0; j < levelTiles[i].length; j++) {
				Vector2 tilePos = new Vector2(
						j * TILE_SIZE, i * TILE_SIZE);
				switch (levelTiles[i][j]) {
					case Level.GRASS_TYPE:
						spriteBatch.draw(
								tilesAtlas.findRegion("grass"),
								tilePos.x, tilePos.y,
								TILE_SIZE, TILE_SIZE);
						break;
				}
			}
		}

		spriteBatch.end();

		debugRenderer.render(world, camera.combined);
	}

}
