package com.mygdx.game.screen;

import com.badlogic.gdx.Game;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Button;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.scenes.scene2d.utils.NinePatchDrawable;
import com.badlogic.gdx.utils.viewport.StretchViewport;

/**
 *
 * @author Martin
 */
public class MainMenuScreen extends ScreenAdapter {

	private SpriteBatch batch;
	private Stage stage;
	private TextureAtlas uiAtlas;
	private Game game;

	public MainMenuScreen(Game game) {
		this.game = game;
		batch = new SpriteBatch();

		stage = new Stage(new StretchViewport(1024, 768), batch);
		Gdx.input.setInputProcessor(stage);

		uiAtlas = new TextureAtlas(Gdx.files.internal("ui.atlas"));
		
		setupUI();
	}

	private void setupUI() {
		FreeTypeFontGenerator freeTypeFontGenerator = new FreeTypeFontGenerator(
				Gdx.files.internal("Gravity-Regular.otf"));

		FreeTypeFontGenerator.FreeTypeFontParameter para
				= new FreeTypeFontGenerator.FreeTypeFontParameter();
		para.size = 36;
		para.color = Color.WHITE;

		TextButton.TextButtonStyle textButtonStyle
				= new TextButton.TextButtonStyle();
		textButtonStyle.up = new NinePatchDrawable(uiAtlas.createPatch(
				"blue"));
		textButtonStyle.down = new NinePatchDrawable(uiAtlas.createPatch(
				"blue_pressed"));
		textButtonStyle.font = freeTypeFontGenerator.generateFont(para);
		textButtonStyle.fontColor = Color.BLACK;

		Button btStart = new TextButton("Start", textButtonStyle);
		btStart.setPosition(100, 100);
		stage.addActor(btStart);

		btStart.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeListener.ChangeEvent event, Actor actor) {
				game.setScreen(new GameScreen());
			}
		});
	}

	@Override
	public void render(float delta) {
		stage.act();
		
		Gdx.gl.glClearColor(1, 1, 1, 1);
		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
		
		stage.draw();
	}

}
